#define INCL_WIN
#define INCL_GPI
#define INCL_DOS
#define INCL_DOSERRORS
#include <os2.h>
#include "mgadive.h"
#include "pci.h"
#include "g200regs.h"

extern PCI_DEV PciDevice;
extern ULONG   MMIOPhys;
extern PBYTE   MMIO;
extern BOOL    IsG400;
extern BOOL    VideoRun;
extern ULONG   BufferSize;
extern ULONG   BufferOffset;
extern ULONG   BufferID;
extern ULONG   CurrentFrame;
extern ULONG   VramHandle;
extern FNHWENTRY *pfnChainedHWEntry;
extern FBINFO  FBInfo;

PBYTE PhysToLin(ULONG addr, ULONG size)
{
    MEMINFO meminfo;

    meminfo.ulPhysAddr = addr;
    meminfo.ulMemLen   = size;
    meminfo.pNextMI    = NULL;
    VHPhysToVirt(&meminfo);
    return (PBYTE)(meminfo.ulVirtAddr);
}

ULONG CheckHardware(void)
{
    ULONG rc;

    /* first, try to find one of supported chips*/
    /* G200 AGP */
    if (pci_find_device(0x102b, 0x521, 0, &PciDevice) == 0) {
        pci_read_dword(&PciDevice, 0x14, &MMIOPhys);
        MMIOPhys &= 0xfffffff0;
        MMIO = PhysToLin(MMIOPhys, 0x4000);
        return RC_SUCCESS;
    }
    /* G200 PCI */
    if (pci_find_device(0x102b, 0x520, 0, &PciDevice) == 0) {
        pci_read_dword(&PciDevice, 0x14, &MMIOPhys);
        MMIOPhys &= 0xfffffff0;
        MMIO = PhysToLin(MMIOPhys, 0x4000);
        return RC_SUCCESS;
    }
    /* G400 */
    if (pci_find_device(0x102b, 0x525, 0, &PciDevice) == 0) {
        pci_read_dword(&PciDevice, 0x14, &MMIOPhys);
        MMIOPhys &= 0xfffffff0;
        MMIO = PhysToLin(MMIOPhys, 0x4000);
        IsG400 = TRUE;
        return RC_SUCCESS;
    }
    /* G550 */
    if (pci_find_device(0x102b, 0x2527, 0, &PciDevice) == 0) {
        pci_read_dword(&PciDevice, 0x14, &MMIOPhys);
        MMIOPhys &= 0xfffffff0;
        MMIO = PhysToLin(MMIOPhys, 0x4000);
        IsG400 = TRUE;
        return RC_SUCCESS;
    }

    /* None of supported chips found */
    return RC_ERROR;
}

void HideVideo(void)
{
    ULONG temp;
    /* if video running, we need to disable BES to hide video window
       when PM goes to background */
    temp = READREG(MMIO, BESCTL);
    temp &= ~1;
    if (VideoRun)
        WRITEREG(MMIO, BESCTL, temp);
}

void ShowVideo(void)
{
    ULONG temp;

    /* if PM go to foreground and video running, enable BES to restore
       video window */
    temp = READREG(MMIO, BESCTL);
    temp |= 1;
    if (VideoRun)
        WRITEREG(MMIO, BESCTL, temp);
}


/* This is main hardware-related function
 * it calculate parameters needed for MGA BES engine.
 * To understand it, request G200 hardware specifications
 * from Matrox
 */
ULONG CalcRegisters(PIMAGEPACK pIn)
{
    ULONG   dataformat;
    ULONG   hifactor, vifactor;        // Horiz, vert inverse scaling factors
    ULONG   hintrep, vintrep;          // Horiz, vert interval representation
    ULONG   offsetleft, offsettop;     // Left and top offset into video window
    ULONG   offsetright, offsetbottom; // Right and bottom offset into video window
    ULONG   srcwidth, destwidth;       // Source and destination width
    ULONG   srcheight, destheight;     // Source and destination height
    ULONG   cropleft, cropright;       // Left and right source crop offsets
    ULONG   croptop, cropbottom;       // Top and bottom source crop offsets
    ULONG   beshsrcst, beshsrcend;     // BES horizontal source start and end
    ULONG   bespitch, base_address;    // BES pitch and source image base address

    dataformat = 1;
    if (pIn->pPutBuf->pColorInfo->fccColorEncoding == FOURCC_Y422)
        dataformat = 2;

    /* Fill out our variables from the input structure */
    srcwidth     = pIn->pPutBuf->ulImgWidth;
    destwidth    = pIn->ulDstXext;
    srcheight    = pIn->pPutBuf->ulImgHeight;
    destheight   = pIn->ulDstYext;
    offsetleft   = -MIN(pIn->ptlDstOrg.x,0);
    offsettop    = -MIN(pIn->ptlDstOrg.y,0);
    offsetright  = (pIn->ptlDstOrg.x+pIn->ulDstXext)-MIN((pIn->ptlDstOrg.x+pIn->ulDstXext), (FBInfo.ulXres-1)); // TODO
    offsetbottom = (pIn->ptlDstOrg.y+pIn->ulDstYext)-MIN((pIn->ptlDstOrg.y+pIn->ulDstYext), (FBInfo.ulYres-1));
    cropleft     = 0; // VG: quick hack pIn->ptlSrcOrg.x;
    cropright    = srcwidth - cropleft - pIn->ulSrcXext;
    croptop      = 0; // VG: quick hack pIn->ptlSrcOrg.y;
    cropbottom   = srcheight - croptop - pIn->ulSrcYext;
    switch (pIn->pPutBuf->pColorInfo->fccColorEncoding) {
      case FOURCC_Y422:
         bespitch     = pIn->pPutBuf->ulBytesPerScan>>1;
         break;
      case FOURCC_Y2X2:
         bespitch     = pIn->pPutBuf->ulBytesPerScan;
         break;
      default:
         return RC_ERROR_INVALID_PARAMETER;
    }

    /* Check pitch */
    if (bespitch > 4092) /* source image too wide*/
        return RC_ERROR_INVALID_PARAMETER;

    /* Determine interval representations */
    hintrep = 0;
    if (!(pIn->ulFlags & IPF_DONT_INTERPOLATE))
        hintrep = 1;
    else
        if (destwidth < srcwidth)
            hintrep = 1;

    vintrep = 0;
    if (!(pIn->ulFlags & IPF_DONT_INTERPOLATE))
        vintrep = 1;
    else
        if (destheight < srcheight)
            vintrep = 1;

    /* Determine inverse scaling factors */
    if ((hintrep>=destwidth)||(vintrep>=destheight)) return RC_ERROR;

    hifactor = ((srcwidth - cropleft - cropright - hintrep) << 14) / (destwidth - hintrep);
    /* Check for scale margin */
    if ((hifactor == 0) || (hifactor >= (1 << 19)))
        return RC_ERROR;

    vifactor = ((srcheight - croptop - cropbottom - vintrep) << 14) / (destheight - vintrep);
    if ((vifactor == 0) || (vifactor >= (1 << 19)))
        return RC_ERROR;

    /* It is not safe to reprogram multiple BES registers while video
       is being displayed */
    HideVideo();

    WRITEREG(MMIO, BESPITCH, bespitch);

    /* Program coordinates of Video window */
    WRITEREG(MMIO, BESHCOORD, ((pIn->ptlDstOrg.x + offsetleft) << 16)
        + (pIn->ptlDstOrg.x + pIn->ulDstXext - offsetright ));
    WRITEREG(MMIO, BESVCOORD, ((pIn->ptlDstOrg.y + offsettop) << 16)
        + (pIn->ptlDstOrg.y + pIn->ulDstYext - offsetbottom ));

    /* Program inverse scaling factors */
    WRITEREG(MMIO, BESHISCAL, hifactor << 2);
    WRITEREG(MMIO, BESVISCAL, vifactor << 2);

    /* Program positioning registers */
    beshsrcst  = (cropleft<<14) + offsetleft * hifactor;
    beshsrcend = beshsrcst + (destwidth - offsetleft - offsetright - 1) * hifactor;
    WRITEREG(MMIO, BESHSRCST, beshsrcst << 2);
    WRITEREG(MMIO, BESHSRCEND, beshsrcend << 2);

    base_address = pIn->ptlSrcOrg.y*FBInfo.ulScanLineBytes+
                   ((croptop+(offsettop*vifactor)>>14)*pIn->pPutBuf->ulBytesPerScan);
    WRITEREG(MMIO, BESA1ORG, base_address);
    WRITEREG(MMIO, BESA1CORG, base_address);
    WRITEREG(MMIO, BESA2ORG, base_address);
    WRITEREG(MMIO, BESA2CORG, base_address);

    WRITEREG(MMIO, BESV1WGHT, (offsettop * vifactor << 2) & 0xFFFF);
    WRITEREG(MMIO, BESV2WGHT, (offsettop * vifactor << 2) & 0xFFFF);

    WRITEREG(MMIO, BESHSRCLST, (srcwidth - 1) << 16);
    WRITEREG(MMIO, BESV1SRCLST, srcheight - 1 - (croptop + (offsettop * vifactor)>>14));
    WRITEREG(MMIO, BESV2SRCLST, srcheight - 1 - (croptop + (offsettop * vifactor)>>14));

    WRITEREG(MMIO, BESGLOBCTL, 0);

    /* Turn on horiz/vert filtering unless disabled */
    if (!(pIn->ulFlags & IPF_DONT_INTERPOLATE))
        WRITEREG(MMIO, BESCTL, 0x0C00);

    /* program color keying registers
     * color keying currently disabled, because I don't know how realize
     * it in DIVE architecture.... */
    *(MMIO+PALWTADD)  = XKEYOPMODE;
    *(MMIO+X_DATAREG) = 1;
    *(MMIO+PALWTADD)  = XCOLMSK0RED;
    *(MMIO+X_DATAREG) = 0xff;
    *(MMIO+PALWTADD)  = XCOLMSK0GREEN;
    *(MMIO+X_DATAREG) = 0xff;
    *(MMIO+PALWTADD)  = XCOLMSK0BLUE;
    *(MMIO+X_DATAREG) = 0xff;
    switch (FBInfo.ulBPP) {
     case 15:
       *(MMIO+PALWTADD)  = XCOLKEY0RED;
       *(MMIO+X_DATAREG) = (TRANS_15BPP & 0x001f);
       *(MMIO+PALWTADD)  = XCOLKEY0GREEN;
       *(MMIO+X_DATAREG) = (TRANS_15BPP & 0x03e0)>>5;
       *(MMIO+PALWTADD)  = XCOLKEY0BLUE;
       *(MMIO+X_DATAREG) = (TRANS_15BPP & 0x7c00)>>10;
       break;
     case 16:
       *(MMIO+PALWTADD)  = XCOLKEY0RED;
       *(MMIO+X_DATAREG) = (TRANS_16BPP & 0x001f);
       *(MMIO+PALWTADD)  = XCOLKEY0GREEN;
       *(MMIO+X_DATAREG) = (TRANS_16BPP & 0x07e0)>>5;
       *(MMIO+PALWTADD)  = XCOLKEY0BLUE;
       *(MMIO+X_DATAREG) = (TRANS_16BPP & 0xf800)>>11;
       break;
     case 24:
     case 32:
       *(MMIO+PALWTADD)  = XCOLKEY0RED;
       *(MMIO+X_DATAREG) = (TRANS_24BPP & 0x00ff);
       *(MMIO+PALWTADD)  = XCOLKEY0GREEN;
       *(MMIO+X_DATAREG) = (TRANS_24BPP & 0x0ff00)>>8;
       *(MMIO+PALWTADD)  = XCOLKEY0BLUE;
       *(MMIO+X_DATAREG) = (TRANS_24BPP & 0xff0000)>>16;
       break;
     default:
       //FIXME: how implement color keying in 8bpp mode?
       *(MMIO+PALWTADD)  = XCOLKEY0RED;
       *(MMIO+X_DATAREG) = (TRANS_8BPP & 0x03);
       *(MMIO+PALWTADD)  = XCOLKEY0GREEN;
       *(MMIO+X_DATAREG) = (TRANS_24BPP & 0x1c)>>2;
       *(MMIO+PALWTADD)  = XCOLKEY0BLUE;
       *(MMIO+X_DATAREG) = (TRANS_24BPP & 0xe0)>>5;

    }

    /* Re-enable BES if necessary */
    ShowVideo();

    return RC_SUCCESS;
}



