
#define INCL_WIN
#define INCL_GPI
#define INCL_DOS
#define INCL_DOSERRORS

#include <os2.h>
#include "gradd.h"
#include "mgadive.h"

extern FNHWENTRY *pfnChainedHWEntry;
extern ULONG VramHandle;
extern BOOL VRAMInit;
extern ULONG BufferSize;
extern ULONG BufferOffset;
extern BOOL Buf1Alloced;
extern BOOL Buf2Alloced;
extern BOOL VideoRun;
extern FBINFO FBInfo;

ULONG VRAMAlloc(PVRAMALLOCIN, PVRAMALLOCOUT);
ULONG VRAMRegister(PVRAMREGISTERIN,PVRAMREGISTEROUT);

/* Basic idea: we provide simple (minimalistic) implementation
 * of GHI_CMD_VRAM. Only one process can allocate up to 2 chunks of VRAM
 * because overlay engine anyway support only one accelerated window
 */
ULONG HWVRAM(PVRAMIN pIn,PVOID pOut) {
ULONG rc=RC_ERROR_INVALID_PARAMETER;
   if (!VRAMInit) return rc;
   switch (pIn->ulFunction) {
     case VRAM_ALLOCATE:
     case VRAM_DEALLOCATE:
     case VRAM_QUERY:
          rc=VRAMAlloc((PVRAMALLOCIN)(pIn->pIn),(PVRAMALLOCOUT)pOut);
          break;
     case VRAM_REGISTER:
     case VRAM_DEREGISTER:
          rc=VRAMRegister((PVRAMREGISTERIN)(pIn->pIn),(PVRAMREGISTEROUT)pOut);
          break;
     default:
          break;
   }
   return rc;
}

ULONG VRAMAlloc(PVRAMALLOCIN pIn, PVRAMALLOCOUT pOut) {
ULONG i,j,rc=RC_ERROR_RESOURCE_NOT_FOUND;
ULONG LineSize;
   if (pIn->ulHandle!=0xdeadbeaf) return RC_ERROR_NO_HANDLE;
   /* rectangle, shared allocation not supported yet !*/
   if (pIn->ulFlags) return  RC_ERROR_INVALID_PARAMETER;
   pOut->ulLength=sizeof(VRAMALLOCOUT);
   switch (pIn->ulFunction) {
     case VRAM_ALLOCATE:
          if (Buf1Alloced && Buf2Alloced) break;
          LineSize=(pIn->ulSize/pIn->ulHeight+7)&~7;

          if (!Buf1Alloced) {
             pOut->ulID=1;
             pOut->ulSize=((pIn->ulWidth*LineSize+FBInfo.ulScanLineBytes-1)/
                          FBInfo.ulScanLineBytes)*FBInfo.ulScanLineBytes;
             pOut->ulScanLineBytes=LineSize;
             pOut->ptlStart.x=0;
             pOut->ptlStart.y=BufferOffset/FBInfo.ulScanLineBytes;
             rc=RC_SUCCESS;
             break;
          }
          if (!Buf2Alloced) {
             pOut->ulID=2;
             pOut->ulSize=((pIn->ulWidth*LineSize+FBInfo.ulScanLineBytes-1)/
                          FBInfo.ulScanLineBytes)*FBInfo.ulScanLineBytes;
             pOut->ulScanLineBytes=LineSize;
             pOut->ptlStart.x=0;
             pOut->ptlStart.y=(BufferOffset+BufferSize)/FBInfo.ulScanLineBytes;
             rc=RC_SUCCESS;
             break;
          }
          /* There are no free VRAM for allocation */
          break;
     case VRAM_DEALLOCATE:
          if (pIn->ulID > 2) break;
          if (pIn->ulID==1) {
             Buf1Alloced=FALSE;
          }else{
             Buf2Alloced=FALSE;
          }
          rc=RC_SUCCESS;
          break;
     case VRAM_QUERY:
          if (Buf1Alloced && Buf2Alloced) {
              pOut->ulSize=0;
          } else {
              pOut->ulSize=BufferSize;
          }
          rc=RC_SUCCESS;
          break;
     default:
          rc=RC_ERROR;
   }
   return rc;
}

ULONG VRAMRegister(PVRAMREGISTERIN pIn,PVRAMREGISTEROUT pOut) {
ULONG i,rc=RC_ERROR_NO_HANDLE;
   pOut->ulLength=sizeof(VRAMREGISTEROUT);
   switch (pIn->ulFlags) {
     case 0:    /* deregistration */
          if (pIn->ulHandle==0xdeadbeaf) {
             VramHandle=0xffffffff;
             HideVideo();
             VideoRun=FALSE;
             rc=RC_SUCCESS;
             }
          break;
     case 1:    /* registration */
          if (VramHandle==0xffffffff) {
             pOut->ulHandle=0xdeadbeaf;
             VramHandle=0xdeadbeaf;
             InitVRAM();
             rc=RC_SUCCESS;
             }
          break;
     default:
          break;
   }
   return rc;
}

/* Get info about current videomode (unfortunately, FBINFO does
   not contain TotalVRAMSize field...) */
ULONG InitVRAM(void) {
PGDDMODEINFO mode;
ULONG i;
  mode=VHAllocMem(sizeof(GDDMODEINFO));
  if (!mode) return RC_ERROR;
  VMIEntry(0,VMI_CMD_QUERYCURRENTMODE,NULL,mode);
  BufferOffset=(mode->ulVertResolution+64)*(mode->ulScanLineSize);
  BufferSize=(mode->ulTotalVRAMSize-BufferOffset)/2;
  Buf1Alloced=FALSE;
  Buf2Alloced=FALSE;
  VRAMInit=TRUE;
  VHFreeMem(mode);
  return RC_SUCCESS;
}
