
#define INCL_WIN
#define INCL_GPI
#define INCL_DOS
#define INCL_DOSERRORS
#include <os2.h>
#include "gradd.h"
#include "mgadive.h"

/* we need to know screen color depth */
extern FBINFO FBInfo;
extern FNHWENTRY *pfnChainedHWEntry;

extern BITBLTINFO bitblt;
extern BMAPINFO   destbmap;
extern PBLTRECT   bltrects;
extern ULONG      cbltrects; /* size of allocated memory */

/* fill some fields of structures, which can be filled only once*/
void OvlHlpInit(void) {
  bitblt.ulLength=sizeof(BITBLTINFO);
  bitblt.ulBltFlags=BF_ROP_INCL_PAT | BF_PAT_SOLID;
  bitblt.ulROP=ROP_PATCOPY;
  bitblt.pSrcBmapInfo=NULL;
  bitblt.pDstBmapInfo=&destbmap;
  bitblt.pPatBmapInfo=NULL;
  bitblt.aptlPatOrg=NULL;
  destbmap.ulLength=sizeof(BMAPINFO);
  destbmap.ulType=BMAP_VRAM;
  switch (FBInfo.ulBPP) {
   case 15:
     bitblt.ulPatFGColor=TRANS_15BPP;
     break;
   case 16:
     bitblt.ulPatFGColor=TRANS_16BPP;
     break;
   case 24:
   case 32:
     bitblt.ulPatFGColor=TRANS_24BPP;
     break;
   default:
     bitblt.ulPatFGColor=TRANS_8BPP;
  }
  bltrects=VHAllocMem(0x1000); /* make one-page allocation */
  cbltrects=0x100;
}



ULONG FillRectangles(PIMAGEPACK pImage) {
ULONG i;
POINTL ptlTemp;
BOOL repaint = (pImage->ulCmdMask & IPC_RECTLS) != 0;
PRECTL prctlRects = pImage->prctlVis;

  if (pImage->cVisRects > cbltrects) {
     /* rare case - to many rectangles, reallocate with page-size granularity */
     VHFreeMem(bltrects);
     cbltrects = (pImage->cVisRects + 0xff) & 0xffffff00;
     bltrects = VHAllocMem(cbltrects << 4);
  }

  bitblt.cBlits = pImage->cVisRects;
  bitblt.abrDst = bltrects;

  for (i = 0; i < pImage->cVisRects; i++) {
    /* convert RECTLs to BLTRECTs and check changes */
    /* also clip rectangles to destination window   */
    ptlTemp.x = MAX(prctlRects[i].xLeft, pImage->ptlDstOrg.x);
    if (bltrects[i].ulXOrg != ptlTemp.x) {
       bltrects[i].ulXOrg = ptlTemp.x;		repaint = TRUE;
       }

    ptlTemp.y = MAX(prctlRects[i].yTop, pImage->ptlDstOrg.y);
    if (bltrects[i].ulYOrg != ptlTemp.y) {
       bltrects[i].ulYOrg = ptlTemp.y;		repaint = TRUE;
       }

    ptlTemp.x = MIN(prctlRects[i].xRight,
		    pImage->ptlDstOrg.x + pImage->ulDstXext) - ptlTemp.x;
    if( ptlTemp.x < 0 )	ptlTemp.x = 0;
    if (bltrects[i].ulXExt != ptlTemp.x) {
       bltrects[i].ulXExt = ptlTemp.x;		repaint = TRUE;
       }

    ptlTemp.y = MIN(prctlRects[i].yBottom,
		    pImage->ptlDstOrg.y + pImage->ulDstYext) - ptlTemp.y;
    if( ptlTemp.y < 0 )	ptlTemp.y = 0;
    if (bltrects[i].ulYExt != ptlTemp.y) {
       bltrects[i].ulYExt = ptlTemp.y;		repaint = TRUE;
       }
  }

  return ( repaint ) ?  RefillRects() : RC_SUCCESS;
}

ULONG RefillRects(void) {

  return pfnChainedHWEntry(0,GHI_CMD_BITBLT,&bitblt,NULL);

}
