
#define INCL_WIN
#define INCL_GPI
#define INCL_DOS
#define INCL_DOSERRORS

#include <os2.h>
#include <string.h>  /* to enable inline memcpy realization */
#include "gradd.h"
#include "mgadive.h"
extern FBINFO FBInfo;
extern FNHWENTRY *pfnChainedHWEntry;
extern BOOL VideoRun;

/* main handler of GHI_CMD_EVENT */
ULONG HWEVENT(PHWEVENTIN pIn) {
  switch (pIn->ulEvent) {
    case EVENT_BACKGROUND:
      HideVideo();
      break;
    case EVENT_FOREGROUND:
       RefillRects();
       ShowVideo();
      break;
    default:
      break;
  }
  /* call other GRADDs in chain */
  return pfnChainedHWEntry(0, GHI_CMD_EVENT, pIn, NULL);
}

/* Initialize EnDive - store framebuffer characteristics*/
ULONG EndiveInit(PFBINFO pFBInfo, PEXTINITOUT pOut ) {

   if(pFBInfo->ulLength != sizeof(FBINFO)) return RC_ERROR;
   memcpy(&FBInfo,pFBInfo,sizeof(FBINFO));
   if(pOut) {
      pOut->ulLength = sizeof(EXTINITOUT);
      pOut->ulFlags = 0;
     }
   if (InitVRAM()) return RC_ERROR;
   OvlHlpInit();
   return RC_SUCCESS;
}

/* Query EnDive characteristics */
ULONG EndiveQuery(PIMAGECAPS pOut) {
ULONG rc=RC_SUCCESS;
   pOut->ulLength=sizeof(IMAGECAPS);
   pOut->ulCaps=IMAGECAPS_STRETCHBLT |
                IMAGECAPS_INTERPOL   |
                IMAGECAPS_NO_SHARED_FB;
   pOut->ulMaxHorz=0xFFFF;
   pOut->ulMaxVert=0xFFFF;
   /* we do not want to Accelerator VRAM*/
   pOut->fAccelMem=FALSE;
   pOut->ulPhysAddrVRAM=0;
   pOut->ulSize=0;
   pOut->ulScanlineBytes=0;
   pOut->ulNumCodecs=0;
   /* first time we only need to signal amount of memory, needed to
      place all supported FOURCC - gre2vman call as second time to
      get actual values */
   if (pOut->pSrcColorInfo &&(pOut->ulNumSrc>=1)) {
   /* there are one source colorspace - Y422 */
      pOut->pSrcColorInfo[0].ulLength=sizeof(COLORINFO);
      pOut->pSrcColorInfo[0].fccColorEncoding=FOURCC_Y422;
      }
    else rc=RC_ERROR; /*no room to place all fourcc */
   pOut->ulNumSrc=1;
   if (pOut->pDstColorInfo &&(pOut->ulNumDst>=1)) {
    /* because we exploit overlay engine, we does not worried about
       destination FOURCC - simply return screen colorspace to enable
       EnDive */
      pOut->pDstColorInfo[0].ulLength=sizeof(COLORINFO);
      pOut->pDstColorInfo[0].fccColorEncoding=FBInfo.fccColorEncoding;
      }
    else rc=RC_ERROR;
   pOut->ulNumDst=1;
   return rc;
}

/* Put new frame to BES */
ULONG EndivePut(PIMAGEPACK pIn) {
ULONG ReinitMask,rc;
  /* First check for parameter errors */
   if( (!pIn->ulDstXext)                      ||
       (!pIn->ulDstYext)                      ||
       (!pIn->ulSrcXext)                      ||
       (!pIn->ulSrcYext)                      ||
       (!pIn->pPutBuf->ulBytesPerScan) )
   {
      return(RC_ERROR);
   }
  ReinitMask=IPC_RECTLS | IPC_DST_SIZE | IPC_DST_POS;
  if (ReinitMask & (pIn->ulCmdMask )) {
     rc=FillRectangles(pIn);
     if (rc) {
        HideVideo();
        VideoRun=FALSE;
        return rc;
     }
  }
  ReinitMask=IPC_SRC_COLOR | IPC_SRC_SIZE |IPC_DST_SIZE |IPC_DST_POS;
  if (ReinitMask & (pIn->ulCmdMask)) {
     /* In such cases we need totally recalculate BES parameters */
     rc=CalcRegisters(pIn);
     if (rc) {
        /* if registers can't be calculated (parameters error as example)
         * hide video window and return error to caller
         */
        HideVideo();
        VideoRun=FALSE;
        return rc;
     }
  }
  VideoRun=TRUE;
  /* enable video output*/
  ShowVideo();
  return RC_SUCCESS;
}

/* main EnDIVE command handler */
ULONG HWExtension(PHWEXTENSION pIn,PVOID pOut){
  switch (pIn->ulXSubFunction) {
    case EXT_CMD_ENDIVE_INIT:
         return EndiveInit(pIn->pXP1,(PEXTINITOUT)pOut);
    case EXT_CMD_ENDIVE_TERM:
         HideVideo();
         return RC_SUCCESS;
    case EXT_CMD_ENDIVE_QUERY:
         return EndiveQuery((PIMAGECAPS)pOut);
    case EXT_CMD_ENDIVE_PUT:
         return EndivePut((PIMAGEPACK)(pIn->pXP1));
    default:
       return RC_ERROR;
  }
}

